//
//  MKMapView+Routing.m
//  Stockland
//
//  Created by Ben Pearson on 22/11/11.
//  Copyright (c) 2011 Inlight Media. All rights reserved.
//

#import "MKMapView+Routing.h"

@implementation MKMapView (Routing)

// from = CLLocationCoordinate2D userCoord = CLLocationCoordinate2DMake(_map.userLocation.location.coordinate.latitude, _map.userLocation.location.coordinate.longitude);
// to = CLLocationCoordinate2DMake(-37.586912, 144.906149)

- (NSString *)showRouteFrom:(CLLocationCoordinate2D)from to:(CLLocationCoordinate2D)to {
	NSString *connectionId = [NSString uuid];
	
	NSString *calcRouteConnectionId = [self calculateRouteFrom:from to:to];
	[[NSNotificationCenter defaultCenter] addObserverForName:[NSString stringWithFormat:REQUEST_SUCCESS_NOTIFICATION_FORMAT,calcRouteConnectionId]
													  object:nil 
													   queue:[NSOperationQueue mainQueue] 
												  usingBlock:^(NSNotification *notification) {
													 //Success													  
													  NSArray* pointStrings = [[notification userInfo] objectForKey:@"points"];
													  // while we create the route points, we will also be calculating the bounding box of our route
													  // so we can easily zoom in on it.
													  MKMapPoint northEastPoint;
													  MKMapPoint southWestPoint;
													  CLLocationCoordinate2D coordinate;
													  // create a c array of points.
													  MKMapPoint* pointArr = malloc(sizeof(CLLocationCoordinate2D) * pointStrings.count);
													  
													  for(int idx = 0; idx < pointStrings.count; idx++) {
														  // break the string down even further to latitude and longitude fields.
														  
														  CLLocation *location = [pointStrings objectAtIndex:idx];
														  
														  CLLocationDegrees latitude  = location.coordinate.latitude;
														  CLLocationDegrees longitude = location.coordinate.longitude;
														  
														  coordinate = CLLocationCoordinate2DMake(latitude, longitude);
														  
														  // create our coordinate and add it to the correct spot in the array
														  //CLLocationCoordinate2D coordinate = CLLocationCoordinate2DMake(latitude, longitude);
														  
														  MKMapPoint point = MKMapPointForCoordinate(coordinate);
														  
														  //
														  // adjust the bounding box
														  //
														  
														  // if it is the first point, just use them, since we have nothing to compare to yet.
														  if (idx == 0) {
															  northEastPoint = point;
															  southWestPoint = point;
														  }
														  else
														  {
															  if (point.x > northEastPoint.x)
																  northEastPoint.x = point.x;
															  if(point.y > northEastPoint.y)
																  northEastPoint.y = point.y;
															  if (point.x < southWestPoint.x)
																  southWestPoint.x = point.x;
															  if (point.y < southWestPoint.y)
																  southWestPoint.y = point.y;
														  }
														  pointArr[idx] = point;
														  
													  }

													  // create the polyline based on the array of points.
													  MKPolyline *routeLine = [MKPolyline polylineWithPoints:pointArr count:pointStrings.count];
													  
													  MKMapRect routeRect = MKMapRectMake(southWestPoint.x, southWestPoint.y, northEastPoint.x - southWestPoint.x, northEastPoint.y - southWestPoint.y);
													  
													  // clear the memory allocated earlier for the points
													  free(pointArr);
													  
													  [[NSNotificationCenter defaultCenter] postNotificationName:[NSString stringWithFormat:REQUEST_SUCCESS_NOTIFICATION_FORMAT,connectionId] 
																										  object:nil 
																										userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
																												  routeLine, @"line", 
																												  MKStringFromMapRect(routeRect), @"rect", 
																												  nil]];	
												  }];
	
	[[NSNotificationCenter defaultCenter] addObserverForName:[NSString stringWithFormat:REQUEST_FAILED_NOTIFICATION_FORMAT,calcRouteConnectionId]
													  object:nil 
													   queue:[NSOperationQueue mainQueue] 
												  usingBlock:^(NSNotification *notification) {
													  // Failed
													  // TODO: pass the error and handle
													  [[NSNotificationCenter defaultCenter] postNotificationName:[NSString stringWithFormat:REQUEST_FAILED_NOTIFICATION_FORMAT,connectionId] 
																										  object:nil 
																										userInfo:nil];
												  }];
	
	return connectionId;
}

- (NSString *)calculateRouteFrom:(CLLocationCoordinate2D)from to:(CLLocationCoordinate2D)to {
	NSString *connectionId = [NSString uuid];
	
	NSString *saddr = [NSString stringWithFormat:@"%f,%f", from.latitude, from.longitude];
	NSString *daddr = [NSString stringWithFormat:@"%f,%f", to.latitude, to.longitude];
	
	NSString *url = [NSString stringWithFormat:@"http://maps.google.com/maps?output=dragdir&saddr=%@&daddr=%@", saddr, daddr];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
	
	AFHTTPRequestOperation *operation = [AFHTTPRequestOperation HTTPRequestOperationWithRequest:request 
																						success:^(id object) {
																					
																							NSString *response = [[NSString alloc] initWithData:object encoding:NSUTF8StringEncoding];
																							
																							NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:@"points:\\\"([^\\\"]*)\\\""
																																								  options:NSRegularExpressionCaseInsensitive
																																									error:nil];
																							NSRange rangeOfFirstMatch = [regex rangeOfFirstMatchInString:response options:0 range:NSMakeRange(0, [response length])];
																							if (NSEqualRanges(rangeOfFirstMatch, NSMakeRange(NSNotFound, 0))) {
																								// ERROR
																							}
																							NSString *encodedPoints = [response substringWithRange:rangeOfFirstMatch];
																							NSArray *points = [self decodePolyLine:[encodedPoints mutableCopy]];
																							[[NSNotificationCenter defaultCenter] postNotificationName:[NSString stringWithFormat:REQUEST_SUCCESS_NOTIFICATION_FORMAT,connectionId] 
																																				object:nil 
																																			  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:points, @"points", nil]];
																						} 
																						failure:^(NSHTTPURLResponse *response, NSError *error) {
																							
																							Log(@"%@", error);																							
																						}];

	NSOperationQueue *queue = [[NSOperationQueue alloc] init];
	[queue addOperation:operation];
	
	return connectionId;
}


- (NSArray *)decodePolyLine:(NSMutableString *)encoded {
    [encoded replaceOccurrencesOfString:@"\\\\" withString:@"\\"
                                options:NSLiteralSearch
                                  range:NSMakeRange(0, [encoded length])];
	
	[encoded replaceOccurrencesOfString:@"points:\"" withString:@""
                                options:NSLiteralSearch
                                  range:NSMakeRange(0, [encoded length])];
	
    NSInteger len = [encoded length];
    NSInteger index = 0;
    NSMutableArray *listCoordinates = [[NSMutableArray alloc] init];
    NSInteger lat=0;
    NSInteger lng=0;
    while (index < len - 1) {
        NSInteger b;
        NSInteger shift = 0;
        NSInteger result = 0;
        do {
			b = [encoded characterAtIndex:index++] - 63;
			result |= (b & 0x1f) << shift;
			shift += 5;
        } while (b >= 0x20);
        NSInteger dlat = ((result & 1) ? ~(result >> 1) : (result >> 1));
        lat += dlat;
        shift = 0;
        result = 0;
        do {
			b = [encoded characterAtIndex:index++] - 63;
			result |= (b & 0x1f) << shift;
			shift += 5;
        } while (b >= 0x20);
        NSInteger dlng = ((result & 1) ? ~(result >> 1) : (result >> 1));
        lng += dlng;
        NSNumber *latitude = [[NSNumber alloc] initWithFloat:lat * 1e-5];
        NSNumber *longitude = [[NSNumber alloc] initWithFloat:lng * 1e-5];
        CLLocation *loc = [[CLLocation alloc] initWithLatitude:[latitude floatValue] longitude:[longitude floatValue]];
		[listCoordinates addObject:loc];
    }
	
    return [NSArray arrayWithArray:listCoordinates];
}


@end